/*****************************************************************************************
 * Copyright (C), 2012
 * SAS Institute Inc., Cary, N.C. 27513, U.S.A.  All rights reserved.
 * 
 *   NAME:    orva_var_sensitivities.sas                                            
 *   PURPOSE: Wrapper for:
 *                        orva_get_model_info
 *                        orva_conf_interval
 *                        orva_extra_loss
 *
 *   SUPPORT: stmajo            
 *
 *   PARAMETERS:
 *
 *    SimPriority_Sev    = the name of the severity priority parameter data set 
 *    SimPriority_Freq   = The name of the frequency dataset containing pointers to the fparm data sets
 *    ccUid              = Which cell the computation is desired for.
 *    tempStatNaked      = Used to get the var numbers for the cell - UPDATED with the result
 *    tempStatInsured    = Used to get the var numbers for the cell 
 *    loss               = The loss dataset with kri's appended 
 *    selectedScenUids   = The list of scenario UIDS that are used in the VaR calculation    
 *    lossKriData        = Internal KRI dataset
 *    lossKriDataC       = Consortium KRI dataset
 *
 ***********************************************************************************/
%macro orva_var_sensitivities( 
                               CCUID              = ,
                               selectedScenUids   = ,
                               SimPriority_Sev    = ,
                               SimPriority_Freq   = ,
                               rareScenSim        = ,
                               lossKriData        = ,
                               lossKriDataC       = , 
                               tempStatNaked      = ,
                               tempStatInsured    = ,
                               SimInterval        = ,
                               maxloss            =
                               );
   %if (%upcase(&ORV_DEBUG.) EQ TRUE) %then %do;
      %put #orva_var_sensitivities(; 
      %put .                        CCUID              = &CCUID.,;
      %put .                        selectedScenUids   = &selectedScenUids.,;
      %put .                        SimPriority_Sev    = &SimPriority_Sev.,;
      %put .                        SimPriority_Freq   = &SimPriority_Freq.,;
      %put .                        rareScenSim        = &rareScenSim.,;
      %put .                        lossKriData        = &lossKriData.,;
      %put .                        lossKriDataC       = &lossKriDataC.,; 
      %put .                        tempStatNaked      = &tempStatNaked.,;
      %put .                        tempStatInsured    = &tempStatInsured.,;
      %put .                        SimInterval        = &SimInterval.,;
      %put .                        Maxloss            = &maxloss.;
      %put .                        );
   %end;

   %let CAN_PROCEED = &ORV_TRUE;
   %let ORV_REASON  = sens_abnormal_error;
   %let CELL_WHERE = ;

   /* --------------------------------------------------- */
   /*  CHECK THAT MODELS AND DISTRIBUTIONS ARE COMPATIBLE */
   /* --------------------------------------------------- */

   /*--- SimPriority_Sev and _Freq must be specified  ---*/
   %if ("&SimPriority_Sev" EQ "") %then %do;
      %let ORV_REASON  =  coni_sim_priority_sev_error;
      %let CAN_PROCEED = &ORV_FALSE.;
      %goto DONE_CHECKING;
   %end; 

   %if ("&SimPriority_Freq" EQ "") %then %do;
      %let ORV_REASON  = coni_sim_priority_freq_error;
      %let CAN_PROCEED = &ORV_FALSE.;
      %goto DONE_CHECKING;
   %end;
  
   /*--- SimPriority_Sev and _Freq must have parameterized models (internal data only, no mixing) ---*/
   %_orv_CheckDataType( 
                       dataset          = &SimPriority_Sev, 
                       CCUID            = &CCUID, 
                       selectedScenUids = &selectedScenUids 
                       );

   /* If rare non-estimated models are found, stop. */
   %if &CAN_PROCEED. eq &ORV_FALSE. %then %do;
      %let ORV_REASON  = coni_not_estimated_model;
      %goto DONE_CHECKING;
   %end;

   %_orv_CheckDataType( 
                       dataset          = &SimPriority_Freq, 
                       CCUID            = &CCUID, 
                       selectedScenUids = &selectedScenUids 
                       );

   /* If non-estimated models are found, stop. */
   %if &CAN_PROCEED. eq &ORV_FALSE. %then %do;
      %let ORV_REASON  = coni_not_estimated_model;
      %goto DONE_CHECKING;
   %end;

   /* Check for Rare Event scenarios- they are not listed in the simpriority datasets */
   %_orv_checkForRareEventScenario( dataset          = &rareScenSim, 
                                    CCUID            = &CCUID, 
                                    selectedScenUids = &selectedScenUids );

   /* If rare events or are found, stop. */
   %if &CAN_PROCEED eq &ORV_FALSE. %then %do;
      %let ORV_REASON  = coni_not_estimated_model;
      %goto DONE_CHECKING;
   %end;

   %orva_get_model_info( 
                        SimPriority_Sev   = &SimPriority_Sev.,
                        SimPriority_Freq  = &SimPriority_Freq.,
                        lossi             = &lossKriData.,
                        lossc             = &lossKriDataC.,
                        sev_model_out     = _vs_model_sev,
                        freq_model_out    = _vs_model_freq,
                        SimInterval       = &SimInterval.,
                        ccuid             = &ccuid.
                        );

   %if &ORV_ABORT. eq &ORV_TRUE. %then %do;
      %put &orvar_error.: %sysfunc(sasmsg(&orvads, gmi_generic_message, noquote, &CCUID));
      %put &orvar_error.: %sysfunc(sasmsg(&orvads, &ORV_REASON, noquote));
      %let CAN_PROCEED = &ORV_FALSE.;
   %end;

%DONE_CHECKING:

   %if &CAN_PROCEED. eq &ORV_FALSE. %then %do;
      %put &orvar_warning.: %sysfunc(sasmsg(&orvads, sens_generic_message, noquote, &CCUID));
      %put &orvar_warning.: %sysfunc(sasmsg(&orvads, &ORV_REASON, noquote));
   %end;

   %orva_conf_interval( 
                        SimPriority_Sev     = &SimPriority_Sev.,  
                        SimPriority_Freq    = &SimPriority_Freq.,  
                        sev_model_dataset   = _vs_model_sev,
                        freq_model_dataset  = _vs_model_freq,
                        ccUid               = &ccUid.,
                        varstat_naked       = &tempStatNaked.,
                        varstat_insured     = &tempStatInsured.,
                        SimInterval         = &SimInterval.,
                        rareScenSim         = &rareScenSim.,
                        selectedScenUids    = &selectedScenUids.,
                        maxloss             = &maxloss.
                        );

   %orva_extra_loss( 
                     ccUid         =      &ccuid.,
                     spriority     =      &SimPriority_Sev.,
                     fpriority     =      &SimPriority_Freq.,
                     sev_model_dataset  = _vs_model_sev,
                     freq_model_dataset = _vs_model_freq,
                     varstat       =      &tempStatNaked.,
                     varstat_ins   =      &tempStatInsured.
                     );

  %orvu_delete_data(data = _vs_model_: );
%mend orva_var_sensitivities;

/*--------------------------------------------------------------------------
  --------------------------------------------------------------------------*/
%macro _orv_CheckDataType(
                          dataset          = , 
                          CCUID            = , 
                          selectedScenUids = 
                          );
   %if(%upcase(&ORV_DEBUG.) EQ TRUE) %then %do;
      %put @_orv_CheckDataType(;
      %put .                    dataset          = &dataset.,; 
      %put .                    CCUID            = &CCUID.,; 
      %put .                    selectedScenUids = &selectedScenUids.;
      %put .                    );
   %end;

   /* IF SOURCE IS NOT I,C,P,T, or R, stop. Also there should only be 1 source type. */
   data _tempDS;
      retain 
            stop 1 
            ;
      set &dataset end = last; 
      where ccuid = &CCUID;

      if ( SOURCE_TYPE in ( "I", "C", "P", "T") ) then stop = 0;

      if last then do;
         if _N_ gt 1 then stop = 1;
		 if stop then do;
            call symput("CAN_PROCEED","&ORV_FALSE.");
	     end;
      end;
   run;

   %let nobs = 0;
   %orvu_getObsCount(_tempDS, nobs);

   %if &nobs eq 0 %then %do;
      %let CAN_PROCEED = &ORV_FALSE.;
   %end;

   %orvu_delete_data(data= _temp: );

   %if &CAN_PROCEED eq &ORV_FALSE. %then %do;
      %return;
   %end;

   /* all SOURCE_TYPES (that aren't Rs) should be the same */
   data _NULL_;
      length TYPE $ 1;
      retain stop 0 TYPE;
      set &dataset end = last; 
      WHERE
            ( SOURCE_TYPE in ( "I", "C", "P", "T") ) 
      AND
            ( ccuid = &CCUID. )
            ;

      if _N_ eq 1 then do;
         TYPE = SOURCE_TYPE;
      end;
      else do;
         if SOURCE_TYPE ne TYPE then stop = 1;
      end;

      if last and stop then do;
         call symput("CAN_PROCEED","&ORV_FALSE.");
      end;
   run;

   %if &CAN_PROCEED eq &ORV_FALSE. %then %do;
      %return;
   %end;

   /* If model has been copied as a model scenario, stop (it's not really model
      based data */
   data _NULL_;
      set &dataset end = last; 
      WHERE
            (SOURCE_TYPE ne "R") 
      AND
            (ccuid = &CCUID.)
            ;

      if last then do;
         call symput("dataset_name",d_name);
      end;
   run;

   %let nobs = 0;
   %orvu_getObsCount(&dataset_name.,nobs);

   %if &nobs ne 0 %then %do;
      data _NULL_;
         set &dataset_name.; 
         where ccuid = &CCUID.;

         if trim(left(REFERENCE)) ne . then do;
            call symput("CAN_PROCEED","&ORV_FALSE.");
         end;
      run;
   %end;

%mend _orv_CheckDataType;

/*---------------------------------------------------------------------------------------*/
/* MACRO: _orv_checkForRareEventScenario                                                 */
/*                                                                                       */
/* PARAMETERS:                                                                           */
/*            dataset          : simulation results data for all rare event sceenarios   */
/*            CCUID            : CCUID for the cell being treated now                    */
/*            selectedScenUids : space separated list of selected rare event scenarios   */
/*                                                                                       */
/*---------------------------------------------------------------------------------------*/
%macro _orv_checkForRareEventScenario( 
                                       dataset          = , 
                                       CCUID            = , 
                                       selectedScenUids = 
                                       );

   %local nobs;

   %if %trim(%left("&selectedScenUids")) ne "" %then %do;
      %if &dataset NE %str() %then %do;
         /* Keep only the data generated for this CCUID */
         /* ANd then only for the selected Scenarios */
         data _coni_RareScenSimThisCCUID_;
            set &dataset;
            WHERE 
                  (CCUID = &CCUID) 
            AND 
                  (SCENARIO_UID in (&selectedScenUids))
                  ;
         run;

         /* Now see how many rows are left.  If there are any, then we have rare  
            event scenario in this cell */
         %let nobs = 0;
         %orvu_getObsCount(_coni_RareScenSimThisCCUID_, nobs);

         %if &nobs > 0 %then %do;
            %let CAN_PROCEED = &ORV_FALSE.;
         %end;
      %end;
   %end;
%mend _orv_checkForRareEventScenario;
